import os

import boto3
from botocore.exceptions import ClientError, EndpointConnectionError


class S3Service:
    def __init__(self):
        self.s3_client = boto3.client('s3')

    def create_new_bucket(self, bucket_name: str, region: str = 'eu-west-2') -> bool:
        """Create an S3 bucket in a specified region

        If a region is not specified, the bucket is created in the S3 default
        region (eu-west-2).

        :param bucket_name: Bucket to create
        :param region: String region to create bucket in, e.g., 'us-west-2'
        :return: True if bucket created, else False
        """
        try:
            location = {'LocationConstraint': region}
            self.s3_client.create_bucket(Bucket=bucket_name,
                                         CreateBucketConfiguration=location)
            return True
        except ClientError as e:
            return e

    def put_file(self, file_name: str, bucket: str, object_name=None) -> bool:
        """Upload a file to an S3 bucket

        :param file_name: File to upload.  Located in folder  /media
        :param bucket: Bucket to upload to
        :param object_name: S3 object name. If not specified then file_name is used
        :return: True if file was uploaded, else False
        """

        # check if file exist
        file_path = os.path.join('./media/', file_name)
        if not os.path.exists(file_path):
            raise FileNotFoundError

        # If S3 object_name was not specified, use file_name
        if object_name is None:
            object_name = os.path.basename(file_name)

        # Upload the file
        try:
            self.s3_client.upload_file(file_path, bucket, object_name)
            return True
        except EndpointConnectionError as e:
            return e
        except ClientError as e:
            return e

    def get_file(self, bucket_name: str, object_name: str) -> bool:
        """ Download file from S3 bucket to folder /media

        :param bucket_name: Name of Bucket
        :param object_name: File to download
        :return: True if file was Dowloaded, else False
        """
        try:
            # create folder (bucket-name), for download files
            if not os.path.exists(f'./media/{bucket_name}'):
                os.mkdir(f'./media/{bucket_name}')

            # check if file exists in bucket, before get(download) from s3
            if not self.check_exist(object_name, bucket_name):
                raise FileNotFoundError

            # download file to folder /media/bucket_name/
            file_path = os.path.join('./media/', bucket_name, object_name)
            with open(file_path, 'wb') as f:
                self.s3_client.download_fileobj(bucket_name, object_name, f)

            # check if file exist (if was saved to /media)
            if not os.path.exists(file_path):
                raise FileNotFoundError
            return True
        except ClientError as e:
            return e

    def check_exist(self, file_name: str, bucket: str) -> bool:
        try:
            # get list of all objects from bucket
            objs = self.s3_client.list_objects(Bucket=bucket)
            list_keys_in_bucket = [obj['Key'] for obj in objs['Contents']]

            # check if object exist
            if file_name in list_keys_in_bucket:
                return True
            else:
                return False
        except ClientError as e:
            return False



from datetime import datetime, timedelta, timezone
import logging.config
import boto3
import pytz
from botocore.exceptions import ClientError, EndpointConnectionError


class CloudwatchService:
    def __init__(self, region='us-east-1'):
        self.region = region
        self._cloud_watch_client = None

    @property
    def cloud_watch_client(self):
        if not self._cloud_watch_client:
            self._cloud_watch_client = boto3.client("cloudwatch", region_name=self.region)
        return self._cloud_watch_client

    def get_cpu_usage(self, instance_id: str):
        """  Function to get CPU usage of EC2 instance

        This function is implemented by 'get_metric_data':
        https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/cloudwatch/client/get_metric_data.html
        :param instance_id: instance_id
        :return:
        """
        try:
            end_time = datetime.now(tz=pytz.UTC)
            start_time = end_time - timedelta(minutes=10)

            response = self.cloud_watch_client.get_metric_data(
                MetricDataQueries=[
                    {
                        'Id': 'cpu_v1',
                        'MetricStat': {
                            'Metric': {
                                'Namespace': 'AWS/EC2',
                                'MetricName': 'CPUUtilization',
                                'Dimensions': [
                                    {
                                        'Name': 'InstanceId',
                                        'Value': instance_id
                                    },
                                ]
                            },
                            'Period': 60,
                            'Stat': 'Average',
                            'Unit': 'Percent'
                        },
                        'ReturnData': True,
                    },
                ],
                StartTime=start_time.strftime('%Y-%m-%dT%H:%M:%SZ'),
                EndTime=end_time.strftime('%Y-%m-%dT%H:%M:%SZ'),
                # NextToken='string',
                ScanBy='TimestampDescending',
                MaxDatapoints=13,
                # LabelOptions={'Timezone': 'timezone'}
            )
            cpu_load_average = response['MetricDataResults'][0]['Values'][0]
            # return response['MetricDataResults']#[0]['Values']
            return '{:.2f}'.format(cpu_load_average)
        except (ClientError, EndpointConnectionError) as e:
            return e

    def get_cpu_usage_2(self,
                        instance_id: str,
                        minutes_timedelta: int = 10):
        """
        Function to get CPU usage
        time stamp must be in ISO 8601 UTC format (for example, 2016-10-03T23:00:00Z).
        This function is implemented by 'get_metric_statistics':
        https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/cloudwatch/client/get_metric_statistics.html
        :param minutes_timedelta: timedelta in minutes, define start time
        :param instance_id: instance_id
        :return: Percents of CPU usage /OR Results List of Timestamps
         example: [{'Timestamp': datetime, 'Average': 0.16, 'Maximum': 0.166, 'Unit': 'Percent'}]
        """
        try:
            end_time = datetime.now(tz=pytz.UTC)
            start_time = end_time - timedelta(minutes=minutes_timedelta)

            response = self.cloud_watch_client.get_metric_statistics(
                Namespace='AWS/EC2',
                MetricName='CPUUtilization',
                Dimensions=[
                    {
                        'Name': 'InstanceId',
                        'Value': instance_id
                    },
                ],
                StartTime=start_time.strftime('%Y-%m-%dT%H:%M:%SZ'),
                EndTime=end_time.strftime('%Y-%m-%dT%H:%M:%SZ'),
                Period=60,
                Statistics=['Maximum', 'Average'],
                Unit='Percent'
            )
            cpu_load_average = response['Datapoints'][0]['Average']
            # return response['Datapoints'] # return all data from response
            return '{:.2f}'.format(cpu_load_average)
        except ClientError as e:
            return e



import boto3
from botocore.exceptions import ClientError, EndpointConnectionError


class Ec2Service:
    def __init__(self, region="eu-west-1"):
        self.region = region
        self._ec2_client = None
        self._account_id = None

    @property
    def ec2(self):
        if not self._ec2_client:
            self._ec2_client = boto3.client("ec2", region_name=self.region)
        return self._ec2_client

    @property
    def account_id(self) -> str:
        """
        Returns details about the IAM user or role whose credentials are used to call the operation.
        $ aws sts get-caller-identity      # command in console to get
        https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/sts/client/get_caller_identity.html
        :return: ID of account (12-digits)
        """
        if not self._account_id:
            self._account_id = boto3.client('sts').get_caller_identity().get('Account')
        return self._account_id

    def get_one_instance_status(self, instance_id: str) -> str:
        response = self.ec2.describe_instance_status(InstanceIds=[instance_id])
        return response['InstanceStatuses'][0]['InstanceState']['Name']

    def describe_one_instance(self, instance_id: str):
        """
        Get info about instance
        :return:
        """
        try:
            return self.ec2.describe_instances(
                InstanceIds=[instance_id, ]
            )
        except ClientError as e:
            return e

    def all_instances(self) -> list:
        """
        Function to return List[ID's] of all instances.
        https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ec2/paginator/DescribeInstances.html
        :return: list of all instances
        """
        response = self.ec2.describe_instances(
            DryRun=False,
            MaxResults=12
        )
        return [item['Instances'][0]['InstanceId'] for item in response['Reservations']]

    def start_instance(self, id_instance: str, dry_run: bool = False) -> str:
        """
        Function to Start one instance
        :param id_instance: id_instance
        :param dry_run: Checks whether you have the required permissions for the action
        :return: status
        """
        response = self.ec2.start_instances(
            InstanceIds=[id_instance],
            # AdditionalInfo='string',
            DryRun=dry_run)
        status = response['StartingInstances'][0]['CurrentState']['Name']
        return status

    def stop_instance(self,
                      instance_id: str,
                      hibernates: bool = True,
                      dry_run: bool = False,
                      force: bool = True) -> dict:
        """
        function to stop One instance
        https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ec2/client/stop_instances.html

        :param instance_id:  instance id
        :param instanceId: str - instanceId
        :param hibernates: Сплячий режим Hibernates the instance if the instance was enabled for hibernation at launch.
        :param dry_run:
        :param force:
        :return: status
        """
        try:
            response = self.ec2.stop_instances(
            InstanceIds=[instance_id],
            Hibernate=False,
            DryRun=dry_run,
            Force=force
        )
            # status = response['StoppingInstances'][0]['Name']
            return response
        except Exception as err:
            return err

    def reboot_instance(self, instance_id: str) -> bool:
        try:
            response = self.ec2.reboot_instances(
                InstanceIds=[instance_id, ],
                DryRun=False
            )
            return True
        except ClientError as e:
            return e

    def create_my_instance(self,
                           ami_id: str = 'ami-0c55b159cbfafe1f0',
                           availib_zone: str = 'eu-west-1a',
                           inst_type: str = 't2.micro') -> dict:
        """
        Function to create one Instance - boto3.client
        https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ec2/service-resource/create_instances.html

        For using   boto3.resource:
        https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ec2/client/run_instances.html

        :param ami_id:  ID of the AMI.
        :param inst_type: type of hard drive, that we want to create
        :param availib_zone: it's not REGION !
        :return:
        """
        try:
            responce = self.ec2.run_instances(
                ImageId=ami_id,  # ID of the AMI.
                InstanceType=inst_type,
                MaxCount=1,
                # ClientToken='string',
                MinCount=1,
                Placement={
                    'AvailabilityZone': availib_zone,
                    'Affinity': 'string',
                    'GroupName': 'string',
                    'PartitionNumber': 123,
                    'HostId': 'string',
                    'Tenancy': 'default',
                    # 'SpreadDomain': 'string', # Reserved for future use
                    'HostResourceGroupArn': 'string',
                    'GroupId': 'string'
                })
            return responce
        except Exception as e:
            raise e



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name='awsservice',
    version='0.0.4',
    description="A small example package for AWS: S3, EC2, CloudWatch",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Vitalik Kost',
    author_email='vitalii.kostyreva@gmail.com',
    packages=find_packages(),
    url='https://github.com/Vitalikys/AWS_s3_app',
    install_requires=[
        'boto3',
        'pytz',
        'botocore',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',
)
